@@ -1,5 +1,7 @@
 from fractions import Fraction
 import pickle
+import copy
+from math import log, exp
 
 general = {
     "det()": ["matrix M", "Returns the determinant of the given matrix.", "det"],
@@ -34,18 +36,27 @@ general = {
     "unityElim()": ["num n", "Essentially deletes the number from print if it is a 1. Used only for dependency purposes.", "unityElim"],
     "inputSystem()": ["num r=1000, num suppress=0, num readback=1", "A program for user-friendly input in order to solve and output solutions to a system of linear equations. r is the rational-denominator limit. Suppress will cancel solution output if ==1, and readback will cancel equation readback output if ==0.", "inputSystem"],
     "help()": ["num repress", "The help function for the matrix module.", "modulehelp"],
-    "create()": ["num p=1", "A program accepting user input to create and store a matrix for use while in the command prompt. p will change the matrix print to rational if ==2, and will suppress printing if ==0.", "create"],
+    "create()": ["num s=0, num p=1", "A program accepting user input to create and store a matrix for use while in the command prompt. String input will only be allowed if s==1. p will change the matrix print to rational if ==2, and will suppress printing if ==0.", "create"],
+    "remove()": ["str key", "Removes the matrix stored under the given key from the matrices index."],
+    "addIndex()": ["matrix M, str key", "Adds a new matrix to the matrices index, under a given key."],
     "listMatrices()": ["num p=1", "Prints all matrices stored in the matrices index. Print defaults to rational, but if p is changed it will be in decimal form.", "listMatrices"],
     "saveMatrices()": ["str filename='matrices.txt'", "A function which saves the matrices index into a text file.", "saveMatrices"],
-    "loadMatrices()": ["str filename='matrices.txt'", "A function which returns an index from a text file. To update the current matrix index, use MatrixModule.index=MatrixModule.loadMatrices(filename).", "loadMatrices"]
+    "loadMatrices()": ["str filename='matrices.txt'", "A function which returns an index from a text file. To update the current matrix index, use MatrixModule.index=MatrixModule.loadMatrices(filename).", "loadMatrices"],
+    "rref()": ["matrix M", "Returns the Reduced Row Echelon Form (RREF) of the given matrix, computed using Gauss-Jordan elimination."],
+    "rInverse()": ["matrix M", "Computes the inverse of a matrix by putting an augmented matrix into RREF using Gauss-Jordan elimination, and returns this inverse."],
+    "elimSystem()": ["matrix A, vector x, vector b, num r=1000", "Computes the solutions to the system Ax=b, using Gauss-Jordan elimination. A must be a square matrix of coefficients, x and b should be column vectors in string and numerical formats respectively. r is the rational-denominator limit. The output will be a list, including both the solution matrix and a series of solution strings.", "elimSystem"],
+    "isMatrix()": ["obj A", "Returns True if the passed object is a matrix, otherwise returns False."],
+    "nthRegression()": ["num r=0", "Computes an nth-degree regression by the least squares method and returns the best fit polynomial. Printing output restricted if r==1."]
 }
 
-def det(matrix):
+
+def det(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the determinant of the given matrix, so long as the input matrix is square.
     m = len(matrix)
     n = len(matrix[0])
     if (m != n):  # Makes sure input is square
-        print ("Failed: Input matrix must be square.")
+        print("Failed: Input matrix must be square.")
         return
     if(m == 1):  # If 1x1 then return inner value
         return matrix[0][0]
@@ -66,7 +77,8 @@ def det(matrix):
     return output
 
 
-def transpose(matrix):
+def transpose(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the transpose of a given matrix.
     m = len(matrix)
     n = len(matrix[0])
@@ -79,7 +91,8 @@ def transpose(matrix):
     return output
 
 
-def zeroMatrix(m, n=0):
+def zeroMatrix(num1, num2=0):
+    m, n = copy.copy(num1), copy.copy(num2)
     # Returns a zero matrix of specified size.
     if(n == 0):
         # If number of columns is not specified then make the matrix square.
@@ -93,7 +106,8 @@ def zeroMatrix(m, n=0):
     return output
 
 
-def identityMatrix(order):
+def identityMatrix(ordr):
+    order = copy.copy(ordr)
     # Returns the identity matrix of specified size.
     output = zeroMatrix(order)  # Begin with a zero matrix
     for i in range(0, order):
@@ -101,7 +115,8 @@ def identityMatrix(order):
     return output
 
 
-def matrify(vector, format="col"):
+def matrify(vec, format="col"):
+    vector = copy.deepcopy(vec)
     # Convert a vector in list (1d list array) format into a matrix (2d list array) for the purposes of vector algebra functions which assume matrix-format. Specify either row (1xn) or column (mx1) vector format for conversion; column is default.
     s = len(vector)
     if (format == "row"):
@@ -120,7 +135,8 @@ def matrify(vector, format="col"):
         return(format + " is not a valid format.")
 
 
-def dot(vec1, vec2):
+def dot(vc1, vc2):
+    vec1, vec2 = copy.deepcopy(vc1), copy.deepcopy(vc2)
     # Returns the dot product of two vectors. Input vectors should both be of the same orientation (row or column) and the same size, and in matrix format.
     m1, n1, m2, n2 = len(vec1), len(vec1[0]), len(vec2), len(vec2[0])
     if (not (m1 == m2 and n1 == n2) or not(m1 == 1 or n1 == 1)):  # Test for size and formatting
@@ -136,7 +152,8 @@ def dot(vec1, vec2):
         return output
 
 
-def multiplyMatrices(matrix1, matrix2):
+def multiplyMatrices(mat1, mat2):
+    matrix1, matrix2 = copy.deepcopy(mat1), copy.deepcopy(mat2)
     # Multiplies two matrices and returns the resulting matrix.
     m1, n1, m2, n2 = len(matrix1), len(
         matrix1[0]), len(matrix2), len(matrix2[0])
@@ -157,7 +174,8 @@ def multiplyMatrices(matrix1, matrix2):
     return output
 
 
-def norm(vector, nrm=2):
+def norm(vec, nr=2):
+    vector, nrm = copy.deepcopy(vec), copy.copy(nr)
     # Returns the nth norm of a vector. If norm is not specified, 2 is used as default, which is the same as the vector's magnitude. The vector must be in matrix list format.
     m = len(vector)
     n = len(vector[0])
@@ -182,7 +200,8 @@ def norm(vector, nrm=2):
         return (value**(1/nrm))
 
 
-def scale(matrix, alpha):
+def scale(mat, a):
+    matrix, alpha = copy.deepcopy(mat), copy.copy(a)
     # Returns the matrix scaled by the given constant value.
     m = len(matrix)
     n = len(matrix[0])
@@ -193,7 +212,8 @@ def scale(matrix, alpha):
     return output
 
 
-def addMatrices(matrix1, matrix2):
+def addMatrices(mat1, mat2):
+    matrix1, matrix2 = copy.deepcopy(mat1), copy.deepcopy(mat2)
     # Adds two matrices and returns the resulting matrix.
     m1, n1, m2, n2 = len(matrix1), len(
         matrix1[0]), len(matrix2), len(matrix2[0])
@@ -207,24 +227,29 @@ def addMatrices(matrix1, matrix2):
     return output
 
 
-def axpy(x, y, a):
+def axpy(mx, my, alpha):
+    x, y, a = copy.deepcopy(mx), copy.deepcopy(my), copy.copy(alpha)
     # Performs the AXPY operation, or single-scaled addition, on two matrices; returning the resulting matrix.
     return addMatrices(scale(x, a), y)
 
 
-def linComb(x, y, a, b):
+def linComb(mx, my, alpha, beta):
+    x, y, a, b = copy.deepcopy(mx), copy.deepcopy(
+        my), copy.copy(alpha), copy.copy(beta)
     # Returns the linear combination of two scaled matrices.
     return addMatrices(scale(x, a), scale(y, b))
 
 
-def rowSwap(matrix, r1, r2):
+def rowSwap(mat, rw1, rw2):
+    matrix, r1, r2 = copy.deepcopy(mat), copy.copy(rw1), copy.copy(rw2)
     # Performs the operation R1<->R2. Inputs should be the 0-indices of the desired rows to swap. Returns resulting matrix.
     output = matrix
     output[r1], output[r2] = output[r2], output[r1]
     return output
 
 
-def rowAddition(matrix, r1, r2):
+def rowAddition(mat, rw1, rw2):
+    matrix, r1, r2 = copy.deepcopy(mat), copy.copy(rw1), copy.copy(rw2)
     # Performs the operation R1+R2->R1. Inputs should be the 0-indices of the desired rows to add. Returns resulting matrix.
     output = matrix
     for i in range(0, len(output[r1])):  # Iterative addition
@@ -232,7 +257,8 @@ def rowAddition(matrix, r1, r2):
     return output
 
 
-def rowScale(matrix, row, alpha):
+def rowScale(mat, rw, alph):
+    matrix, row, alpha = copy.deepcopy(mat), copy.copy(rw), copy.copy(alph)
     # Performs the operation AR1->R1. Input the 0-index of the desired row and the scalar Alpha. Returns resulting matrix.
     output = matrix
     for i in range(0, len(output[row])):  # Iterative scaling
@@ -240,7 +266,9 @@ def rowScale(matrix, row, alpha):
     return output
 
 
-def scaledRowAddition(matrix, r1, r2, alpha):
+def scaledRowAddition(mat, rw1, rw2, alph):
+    matrix, r1, r2, alpha = copy.deepcopy(mat), copy.copy(
+        rw1), copy.copy(rw2), copy.copy(alph)
     # Performs the operation R1+AR2->R1. Inputs should be the 0-indices of the desired rows to add, as well as the scalar Alpha. Returns resulting matrix.
     output = matrix
     for i in range(0, len(output[r1])):  # Iterative scaled addition
@@ -248,7 +276,8 @@ def scaledRowAddition(matrix, r1, r2, alpha):
     return output
 
 
-def hAppend(matrix1, matrix2):
+def hAppend(mat1, mat2):
+    matrix1, matrix2 = copy.deepcopy(mat1), copy.deepcopy(mat2)
     # Appends matrix2 to the right of matrix, and returns the resulting matrix.
     m1 = len(matrix1)
     m2 = len(matrix2)
@@ -264,7 +293,8 @@ def hAppend(matrix1, matrix2):
     return output
 
 
-def trace(matrix):
+def trace(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the trace of a square matrix (the sum of elements along the main diagonal).
     m, n = len(matrix), len(matrix[0])
     if (m != n):  # Confiems that matrix is square.
@@ -275,7 +305,8 @@ def trace(matrix):
     return output
 
 
-def mPrint(matrix, s=1, l=0):
+def mPrint(mat, s=1, l=0):
+    matrix = copy.deepcopy(mat)
     # Prints the values of a matrix or vector in a specific format designed for readability.
     # There will be s spaces between each entry of a row, and l lines between rows.
     m, n = len(matrix), len(matrix[0])
@@ -292,7 +323,8 @@ def mPrint(matrix, s=1, l=0):
     return
 
 
-def ratMPrint(matrix, s=1, l=0):
+def ratMPrint(mat, s=1, l=0):
+    matrix = copy.deepcopy(mat)
     # Prints the values of a matrix or vector in a specific format designed for readability, in rational (fractional) format rather than decimal.
     # There will be s spaces between each entry of a row, and l lines between rows.
     m, n = len(matrix), len(matrix[0])
@@ -303,7 +335,7 @@ def ratMPrint(matrix, s=1, l=0):
             spaces += " "
         for j in range(0, n):
             str += ("%s%s") % (spaces, Fraction(matrix[i]
-                                        [j]).limit_denominator())
+                                                [j]).limit_denominator())
         print(str)
         for k in range(0, l):
             print(" ")
@@ -317,7 +349,8 @@ def line(n=1):
     return
 
 
-def cutLeftSquare(matrix):
+def cutLeftSquare(mat):
+    matrix = copy.deepcopy(mat)
     # Cuts a non square matrix into a square, focused on the upper left side, by removing rows or columns from the bottom or right as needed.
     m, n = len(matrix), len(matrix[0])
     output = []
@@ -333,7 +366,8 @@ def cutLeftSquare(matrix):
     return output
 
 
-def cutRightSquare(matrix):
+def cutRightSquare(mat):
+    matrix = copy.deepcopy(mat)
     # Cuts a non square matrix into a square, focused on the upper right side, by removing rows or columns from the bottom or left as needed.
     m, n = len(matrix), len(matrix[0])
     output = []
@@ -350,7 +384,8 @@ def cutRightSquare(matrix):
     return output
 
 
-def cofactor(matrix):
+def cofactor(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the cofactor matrix associated with the given matrix.
     m, n = len(matrix), len(matrix[0])
     if (m != n):
@@ -371,17 +406,20 @@ def cofactor(matrix):
     return output
 
 
-def adjugate(matrix):
+def adjugate(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the adjugate (adjoint) matrix associated with the given matrix, which is the transpose of the associated cofactor matrix.
     return transpose(cofactor(matrix))
 
 
-def inverse(matrix):
+def inverse(mat):
+    matrix = copy.deepcopy(mat)
     # Returns the inverse of a matrix, calculated using the determinant and adjugate of that matrix.
     return scale(adjugate(matrix), 1/det(matrix))
 
 
-def invSystem(A, x, b, rat=1000):
+def invSystem(mA, vx, vb, rat=1000):
+    A, x, b = copy.deepcopy(mA), copy.deepcopy(vx), copy.deepcopy(vb)
     # Calculates the solutions to the system Ax=b, using the method of inverse multiplication.
     # Returns a list, beginning with the solution matrix, followed by a series of strings expressing these solutions.
     # Only works on square systems, where there are as many coefficients in a row of A as there are unknowns in x.
@@ -411,7 +449,8 @@ def invSystem(A, x, b, rat=1000):
     return output
 
 
-def solutionPrint(solutions):
+def solutionPrint(sols):
+    solutions = copy.deepcopy(sols)
     # Prints the solutions to a system of equations generated by another function in a readable way.
     m = len(solutions[1])
     for i in range(0, m):
@@ -419,7 +458,8 @@ def solutionPrint(solutions):
     return
 
 
-def sysSign(num):
+def sysSign(n):
+    num = copy.copy(n)
     if(num >= 0):
         return "+"
     elif(num < 0):
@@ -428,40 +468,219 @@ def sysSign(num):
         return ""
 
 
-def unityElim(num):
+def unityElim(n):
+    num = copy.copy(n)
     if(num == 1):
         return ""
     else:
         return num
 
 
-def inputSystem(rat=1000, suppress=0, readback=0):
+def help(rep=0):
+    modulehelp(rep)
+    return
+
+
+def modulehelp(rep=0):
+    if(rep == 0):
+        print("This is the Matrix Module V0.0.11, a package created by Elliott Walker. Type 'funcs' for a list of available functions, 'details' for a more detailed list including parameters, or 'exit' to exit this help function.")
+    else:
+        print("Available commands: 'funcs', 'details', 'exit'")
+    inp = input()
+    if(inp == "func" or inp == "functions" or inp == "list" or inp == "funcs"):
+        print("Available functions under Matrix Module:")
+        for key in general.keys():
+            print(key)
+    elif(inp == "details" or inp == "det" or inp == "dets" or inp == "more"):
+        print("Available functions under Matrix Module (with parameters and descriptions):")
+        line()
+        for key in general.keys():
+            print("%s  Parameters: %s;  Description: %s" %
+                  (key, general[key][0], general[key][1]))
+            line()
+    elif(inp == "exit"):
+        return
+    help(1)
+    return
+
+
+index={}
+
+def create(string=0, p=1):
+    pr=copy.copy(p)
+    print("Enter size of matrix, e.g. '3x3':")
+    sizeIn=input()
+    sizeArr=sizeIn.split("x")
+    if (len(sizeArr) != 2):
+        print("Program failure: Improper formatting of input.")
+        return
+    m=int(sizeArr[0])
+    n=int(sizeArr[1])
+    print("Enter a key to store this %d x %d matrix in." % (m, n))
+    key=input()
+    if(key in index.keys()):
+        print("Program failure: Key already exists in index.")
+        return
+    index[key]=zeroMatrix(m, n)
+    print("Matrix stored in index['%s']." % (key))
+    for i in range(m):
+        for j in range(n):
+            if(string == 1):
+                index[key][i][j]=str(
+                    input("Input item at position %d, %d: " % (i+1, j+1)))
+            else:
+                index[key][i][j]=int(
+                    input("Input value at position %d, %d: " % (i+1, j+1)))
+    print("Matrix created.")
+    if pr == 1:
+        mPrint(index[key])
+    elif pr == 2:
+        ratMPrint(index[key])
+    return
+
+
+def remove(key):
+    if key in index:
+        del index[key]
+    return
+
+def addIndex(matrix, key):
+    if (matrix.isMatrix == True):
+        if (key in index.keys()):
+            print("Program failure: Key already exists in index.")
+        else:
+            index[key]=matrix
+    else:
+        print("Program failure: Object passed is not a matrix.")
+    return
+
+def listMatrices(rat=1):
+    print("All stored matrices:")
+    for key in index.keys():
+        print("")
+        print(" At index['%s']: " % (key))
+        if rat == 1:
+            ratMPrint(index[key])
+        else:
+            mPrint(index[key])
+    print("")
+    print("Access a stored matrix using MatrixModule.index['key'].")
+    return
+
+def saveMatrices(filename="matrices.txt"):
+    # Saves the matrices index to a file using the pickle dump method.
+    file=open(filename, "wb")
+    file.truncate(0)
+    pickle.dump(index, file)
+    file.close()
+    return
+
+
+def loadMatrices(filename="matrices.txt"):
+    # Loads a matrices index from a pickle-binary text file.
+    file=open(filename, "rb")
+    index=pickle.load(file)
+    file.close()
+    return index
+
+
+def rref(mat):
+    matrix=copy.deepcopy(mat)
+    # Through Gauss-Jordan elimination, reduces a matrix to reduced row echelon form (RREF) and returns the resulting matrix. Only works if the matrix is square or fat: tall matrices (m>n) will not work with the given algorithm.
+    m, n=len(matrix), len(matrix[0])
+    if (m > n):
+        print("Failure: tall matrices (m>n) cannot reduce through given method.")
+        return
+    output=matrix
+    # Row Reduction
+    for i in range(m):  # Iterate through each row.
+        # If the row's lead value isn't 1, then scale the row by dividing by that value so that it is.
+        if (output[i][i] != 1):
+            output=rowScale(output, i, 1/output[i][i])
+        # Now iterate through all of the other rows, and perform scaled addition to make the values in the column of the current leading 1 become zeroes.
+        for j in range(i+1, m):
+            output=scaledRowAddition(output, j, i, -output[j][i])
+    # Back Substitution
+    # Iterate through the rows, excluding the last one, backwards.
+    for i in range(m-1, -1, -1):
+        # For each element to the right of the diagonal, subtract the row with a leading 1 in that column (scaled by the value of the aforementioned element) from the current row.
+        for j in range(m-1, i, -1):
+            output=scaledRowAddition(output, i, j, -output[i][j])
+    return output
+
+
+def rInverse(mat):
+    matrix=copy.deepcopy(mat)
+    # Computes the inverse of a matrix using Gauss-Jordan elimination on an augmented matrix, and returns this resulting inverse matrix.
+    if (det(matrix) == 0 or len(matrix) != len(matrix[0])):
+        return("Matrix must be invertible.")
+    return cutRightSquare(rref(hAppend(matrix, identityMatrix(len(matrix)))))
+
+
+def elimSystem(mA, vx, vb, rat=1000):
+    A, x, b=copy.deepcopy(mA), copy.deepcopy(vx), copy.deepcopy(vb)
+    # Calculates the solutions to the system Ax=b, using the method of Gaussian elimination.
+    # Returns a list, beginning with the solution matrix, followed by a series of strings expressing these solutions.
+    # Only works on square systems, where there are as many coefficients in a row of A as there are unknowns in x.
+    # A should be an mxm matrix. x and b both should be mx1 column vectors in matrix format.
+    # A should be the matrix of coefficients, as numerical values. x should be the vector of unknowns, as string values. b should be the vector of constants, as numerical values.
+    output=[]
+    if not(len(A) == len(A[0]) == len(x) == len(b)):  # Confirms compatiblity
+        output.append("Matrices are not of compatible sizes.")
+    else:
+        m=len(x)
+        # Perform the actual calculation
+        solMtx=rref(hAppend(A, b))
+        output.append(solMtx)  # Primary element of list inserted
+        strList=[]
+        # Will provide the solution strings in rational fraction format, unless passed a 'rat' value of 0.
+        for i in range(0, m):  # Iterates through each unknown
+            if(rat != 0):
+                # Creates readable string in rational fraction form for the solution to this unknown
+                st="%s = %s" % (x[i][0], Fraction(
+                    solMtx[i][len(x)]).limit_denominator(rat))
+            else:
+                # Creates readable string for the solution to this unknown
+                st="%s = %s" % (x[i][0], solMtx[i][len(x)])
+            strList.append(st)  # Appends this solution string into the list
+        # Appends the solution string list into the output
+        output.append(strList)
+    return output
+
+
+def inputSystem(rat=1000, suppress=0, readback=1):
     # Accepts input for a much more user-friendly method of solving a system of linear equations.
+    print("Would you like to solve using Gauss-Jordan elimination (1), or computational inverse (2)?")
+    methodIn=input()
+    if (int(methodIn) == 2):
+        method="Inv"
+    else:
+        method="GJ"
     print("How many unknowns?")
-    m = int(input())
-    x = zeroMatrix(m, 1)
+    m=int(input())
+    x=zeroMatrix(m, 1)
     for i in range(0, m):
         print("What is the unknown at index %d?" % (i))
-        x[i][0] = input()
+        x[i][0]=input()
     print("When inputting coefficients separate them with spaces. For example, input 4x+3y=9 as '4 3 9'.")
-    A = zeroMatrix(m)
-    b = zeroMatrix(m, 1)
+    A=zeroMatrix(m)
+    b=zeroMatrix(m, 1)
     for j in range(0, m):
         print("What are the coefficients in the equation of index %d?" % (j))
-        B = input().split()
+        B=input().split()
         if (len(B) != (m+1)):
             print("Program failure: Incorrect number of coefficients entered.")
             return
         for k in range(0, m+1):
             if (k == m):
-                b[j][0] = int(B[k])
+                b[j][0]=int(B[k])
             else:
-                A[j][k] = int(B[k])
+                A[j][k]=int(B[k])
     if (readback == 1):
         line()
         print("Your input:")
         for row in range(0, m):
-            equ = ""
+            equ=""
             for val in range(0, m+1):
                 if (val == m):
                     equ += ("= %d" % (b[row][0]))
@@ -474,7 +693,10 @@ def inputSystem(rat=1000, suppress=0, readback=0):
     if (det(A) == 0):
         print("System dependent - method failed.")
         return
-    output = invSystem(A, x, b, rat)
+    if (method == "Inv"):
+        output=invSystem(A, x, b, rat)
+    else:
+        output=elimSystem(A, x, b, rat)
     if (suppress == 0):
         line()
         print("Solution:")
@@ -483,80 +705,93 @@ def inputSystem(rat=1000, suppress=0, readback=0):
         input("Press return to exit.")
     return(output)
 
-
-def help(rep=0):
-    modulehelp(rep)
-    return
-
-def modulehelp(rep=0):
-    if(rep == 0):
-        print("This is the Matrix Module V0.0.4, a package created by Elliott Walker. Type 'funcs' for a list of available functions, 'details' for a more detailed list including parameters, or 'exit' to exit this help function.")
+def isMatrix(obj):
+    if (type(obj)==list):
+        for i in range(len(obj)):
+            if (obj[i].type() != list):
+                return False
+        return True
     else:
-        print("Available commands: 'funcs', 'details', 'exit'")
-    inp = input()
-    if(inp == "func" or inp == "functions" or inp == "list" or inp == "funcs"):
-        print("Available functions under Matrix Module:")
-        for key in general.keys():
-            print(key)
-    elif(inp == "details" or inp == "det" or inp == "dets" or inp == "more"):
-        print("Available functions under Matrix Module (with parameters and descriptions):")
-        for key in general.keys():
-            print("%s  Parameters: %s;  Description: %s" %
-                  (key, general[key][0], general[key][1]))
-    elif(inp == "exit"):
-        return
-    help(1)
-    return
+        return False
 
-index={}
-
-def create(pr=1):
-    print("Enter size of matrix, e.g. '3x3':")
-    sizeIn = input()
-    sizeArr = sizeIn.split("x")
-    if (len(sizeArr) != 2):
-        print("Program failure: Improper formatting of input.")
-        return
-    m = int(sizeArr[0])
-    n = int(sizeArr[1])
-    print("Enter a key to store this %d x %d matrix in." % (m, n))
-    key = input()
-    if(key in index.keys()):
-        print("Program failure: Key already exists in index.")
-        return
-    index[key] = zeroMatrix(m, n)
-    print("Matrix stored in 'index[%s]'." % (key))
-    for i in range(m):
-        for j in range(n):
-            index[key][i][j] = int(
-                input("Input value at position %d, %d: " % (i+1, j+1)))
-    print("Matrix created.")
-    if pr == 1:
-        mPrint(index[key])
-    elif pr == 2:
-        ratMPrint(index[key])
-    return
-
-def listMatrices(rat=1):
-    print("All stored matrices:")
-    for key in index.keys():
-        print(" At index[%s]: " % (key))
-        if rat == 1:
-            ratMPrint(index[key])
+def nthRegression(restrict=0,inMatrix=["empty"],ndeg=0):
+    # Performs least-squares regression in order to find the best fitting nth-degree polynomial. For n > 6 the computation may take a long time / not work, as computational expense goes up exponentially with increasing n (inverse of (n+1)x(n+1) matrix must be taken...)
+    npoints = 0
+    if(inMatrix==["empty"]):
+        print("What degree regression would you like to perform?")
+        ndeg = int(input())
+        print("Would you like to use Matrix input (1) or Manual input (2)?")
+        inmode = input()
+        dataMt = []
+        if (int(inmode) == 1):
+            # I don't have anything to handle input matrices that are too small at the moment.
+            print("Enter index of the dataset matrix to input. For example, to access the matrix at location index['matA'], simply type 'matA' (no quotes).")
+            indx = input()
+            dataMt = index[str(indx)]
         else:
-            mPrint(index[key])
-        print("")
-    return
-
-def saveMatrices(filename="matrices.txt"):
-    file=open(filename, "wb")
-    file.truncate(0)
-    pickle.dump(index, file)
-    file.close()
-    return
-
-def loadMatrices(filename="matrices.txt"):
-    file=open(filename, "rb")
-    index=pickle.load(file)
-    file.close()
-    return index
+            print("How many datapoints would you like to input? You must have at least %d datapoints. When entering datapoints use the format x,y."%(ndeg))
+            npoints = int(input())
+            if(npoints < ndeg):
+                print("Not enough datapoints. Failed.")
+                return
+            for i in range(npoints):
+                print("Enter datapoint at index %d:"%(i))
+                ptIn = input()
+                ptArr = ptIn.split(",")
+                if(len(ptArr) != 2):
+                    print("Program failure: Improper formatting of input.")
+                    return
+                dataMt.append([float(ptArr[0]),float(ptArr[1])])
+    else:
+        dataMt = inMatrix
+        npoints = len(dataMt)
+    xset = []
+    yset = []
+    for i in range(npoints):
+        yset.append([dataMt[i][1]])
+        rowMat = []
+        xcur = dataMt[i][0]
+        for j in range(ndeg+1):
+            rowMat.append(xcur**j)
+        xset.append(rowMat)
+    coeffs = multiplyMatrices(multiplyMatrices(inverse(multiplyMatrices(transpose(xset),xset)),transpose(xset)),yset)
+    outpoly = "y = "
+    for i in range(ndeg+1):
+        outpoly += ("%sx^%d" % (Fraction(coeffs[i][0]).limit_denominator(), i)) # Readable format y = ax^0 + bx^1 + cx^2 + ...
+        if(i < ndeg):
+            outpoly += " + "
+    if(restrict != 1):
+        print("outpoly")
+    return [coeffs,outpoly]
+    
+def pwrRegression(restrict=0):
+    # Computes the closest power-regression (form of y=ax^b) fit to a dataset using logarithmic least-squares
+    print("Would you like to use Matrix input (1) or Manual input (2)?")
+    inmode = input()
+    lnMt = []
+    if (int(inmode) == 1):
+        # I don't have anything to handle input matrices that are too small at the moment.
+        print("Enter index of the dataset matrix to input. For example, to access the matrix at location index['matA'], simply type 'matA' (no quotes).")
+        indx = input()
+        lnMt = index[str(indx)]
+    else:
+        print("How many datapoints would you like to input? You must have at least 3 datapoints. When entering datapoints use the format x,y.")
+        npoints = int(input())
+        if(npoints < 3):
+            print("Not enough datapoints. Failed.")
+            return
+        for i in range(npoints):
+            print("Enter datapoint at index %d:"%(i))
+            ptIn = input()
+            ptArr = ptIn.split(",")
+            if(len(ptArr) != 2):
+                print("Program failure: Improper formatting of input.")
+                return
+            lnMt.append([log(float(ptArr[0])),log(float(ptArr[1]))]) # Converts coordinates to (log(x),log(y)) to exploit fact that log(ax^b) = log(a) + b*log(x), which is linear.
+    # Need to take user input, like for nthRegression.
+    lnMt_reg = nthRegression(1,lnMt,1)[0] # Performs linear regression (n=1) on logarithmic dataset
+    lnMt_reg[0][0] = exp(lnMt_reg[0][0]) # Accounts for fact that "y-intercept" value is still in natural logarithm form by performing exponentiation
+    outeq = "y = %sx^(%s)" % (lnMt_reg[0][0],lnMt_reg[1][0]) # Readable format y = ax^(b)
+    if(restrict != 1):
+        print(outeq)
+    return [lnMt_reg,outeq]
\ No newline at end of file
